<?php
// $Id: zoomify.inc,v 1.2.2.3.4.1 2008/07/26 12:54:05 kratib Exp $
/**
 * @file Utility and API functions for zoomify.module
 */

function _zoomify_images($node) {
  $images = array();
  foreach (module_implements('images') as $module) {
    $these_images = module_invoke($module, 'images', $node);
    if (!empty($these_images)) {
      $images = $images + $these_images;
    }
  }
  return $images;
}

function _zoomify_basepath() {
  $basepath = file_directory_path().'/zoomify';
  if (!file_check_directory($basepath, FILE_CREATE_DIRECTORY)) {
    $msg = 'Could not create path %path';
    $arg = array('%path' => $basepath);
    drupal_set_message(t($msg, $arg), 'error');
    watchdog('zoomify', $msg, $arg, WATCHDOG_ERROR);
  }
  return $basepath;
}

function _zoomify_nodepath($node) {
  return file_create_path(_zoomify_basepath().'/'.$node->nid);
}

function _zoomify_filepath($node, $fid) {
  return file_create_path(_zoomify_nodepath($node).'/'.$fid);
}

function _zoomify_identical($node, $fid, $filepath) {
  $md5 = md5_file($filepath);
  $md5_path = _zoomify_filepath($node, $fid).'/Image.md5';
  if (!file_exists($md5_path)) return FALSE;
  $md5_existing = file_get_contents($md5_path);
  return $md5_existing == $md5;
}

function _zoomify_insert_node($node) {
  $images = _zoomify_images($node);
  if (empty($images)) return;
  foreach ($images as $fid => $filepath) {
    _zoomify_process($node, $fid, $filepath);
  }
}

function _zoomify_process($node, $fid, $filepath) {
  // TODO: Let user adjust maximum time for scripts or provide better mechanism.
  set_time_limit(0);
  $info = image_get_info($filepath);
  if ($info === FALSE) {
    $msg = 'Could not find image info for file %path.';
    $arg = array('%path' => $filepath);
    watchdog('zoomify', $msg, $arg, WATCHDOG_ERROR);
    drupal_set_message(t($msg, $arg), 'error');
    return;
  }
  if (($info['width'] < variable_get('zoomify_minimum_width', 1024))
   && ($info['height'] < variable_get('zoomify_minimum_height', 768))) return;

  require_once(drupal_get_path('module', 'zoomify').'/ZoomifyFileProcessor.php');
  $zoomify = new ZoomifyFileProcessor();
  $zoomify->ZoomifyProcess($filepath);
  $msg = 'Created tiles for %path.';
  $arg = array('%path' => $filepath);
  drupal_set_message(t($msg, $arg), 'status');
  watchdog('zoomify', $msg, $arg, WATCHDOG_NOTICE);

  if (!file_check_directory(_zoomify_nodepath($node), FILE_CREATE_DIRECTORY)) {
    $msg = 'Could not create path %path.';
    $arg = array('%path' => _zoomify_nodepath($node));
    drupal_set_message(t($msg, $arg), 'error');
    watchdog('zoomify', $msg, $arg, WATCHDOG_ERROR);
  }
  
  rename($zoomify->_v_saveToLocation, _zoomify_filepath($node, $fid));
  $md5 = md5_file($filepath);
  $md5_path = _zoomify_filepath($node, $fid).'/Image.md5';
  file_put_contents($md5_path, $md5);
}

function _zoomify_delete_node($node) {
  if (is_dir(_zoomify_nodepath($node))) {
    rrmdir(_zoomify_nodepath($node));
    $msg = 'Tiles deleted for node %nid.';
    $arg = array('%nid' => $node->nid);
    watchdog('zoomify', $msg, $arg, WATCHDOG_NOTICE);
    drupal_set_message(t($msg, $arg), 'status');
  }
}

function _zoomify_update_node($node) {
  $old_images = file_scan_directory(_zoomify_nodepath($node), '.*', array('.', '..'), 0, FALSE);
  $images = _zoomify_images($node);
  foreach ($images as $fid => $filepath) {
    if (!_zoomify_identical($node, $fid, $filepath)) {
      rrmdir(_zoomify_filepath($node, $fid));
      _zoomify_process($node, $fid, $filepath);
    }
  }
  foreach ($old_images as $old_image) {
    if (!array_key_exists($old_image->basename, $images) && is_dir($old_image->filename)) {
      rrmdir($old_image->filename);
      $msg = 'Tiles deleted for obsolete file %fid.';
      $arg = array('%fid' => $old_image->basename);
      watchdog('zoomify', $msg, $arg, WATCHDOG_NOTICE);
      drupal_set_message(t($msg, $arg), 'status');
    }
  }
}

